#!/bin/bash
# Copyright (c) 2022 xFusion Digital Technologies Co., Ltd. All rights reserved.
# Description: delete logfile by day or size according to configuration
# File Name: esyslog_delete_by_day_or_size.sh
# Author: miaokaibo
# Create: 2022-04-15

source /opt/esyslog/esyslog_log_source_dump.sh

LOG_SAVE_DAYS="0"
LOG_SAVE_SIZE="0"
LOG_BAK_DIR=""
LOG_PACKAGE="/var/log/logdump*.tar.gz"
CUR_SCRIPT_NAME=${BASH_SOURCE}
################################################################################
# Function Name: fn_delete_by_day
# Description: delete file by day
# Parameter: none.
################################################################################
function fn_delete_by_day()
{
	local year_month_day="0"
	local tmp_year_month_day="0"
	local storage_days="0"


	date_list=`ls ${LOG_BAK_DIR} | grep "^logdump-.*-.*\.tar\.gz$" | sort -t- -r -nk2 | cut -d '-' -f2,3`
	for date in ${date_list}
	do
		year_month_day="`echo ${date} | cut -d '-' -f2`"
		year_month_day="`echo ${year_month_day:0:8}`"
		if [ ${tmp_year_month_day} -ne 0 ];then
			((time_diff=${tmp_year_month_day}-${year_month_day}))
			if [ ${time_diff} -ne 0 ];then
				((storage_days=${storage_days}+1))
			fi
		fi

		tmp_year_month_day=${year_month_day}
		if [ ! -z ${LOG_SAVE_DAYS} ] && [ "${storage_days}" -ge "${LOG_SAVE_DAYS}" ];then
			rm -rf "${LOG_BAK_DIR}/logdump-${date}"
		fi
	done
	return 0
}

################################################################################
# Function Name: fn_delete_by_size
# Description: delete file by size
# Parameter: none
################################################################################
function fn_delete_by_size()
{
	local total_size=0
	local old_file=""
	local del_file=""
	local logdump_dir_size=0
	local log_package_size=0

	ls ${LOG_BAK_DIR} | grep "logdump" | grep "tar.gz" >/dev/null
	if [[ $? -eq 0 ]];then
		log_package_size=`du -cm ${LOG_PACKAGE} | grep total | cut -f1`
	fi

	if [ -d ${LOG_BAK_DIR}/logdump ];then
		pushd ${LOG_BAK_DIR}/logdump > /dev/null
		logdump_dir_size=`du -sm | awk '{print $1}'`
		popd > /dev/null
	fi

	((total_size=${log_package_size}+${logdump_dir_size}))
	while [ ${total_size} -gt ${LOG_SAVE_SIZE} ]
	do
		old_file=`ls ${LOG_BAK_DIR} | grep "^logdump-.*-.*\.tar\.gz$" | sort -t- -k2n | cut -d '-' -f2,3 | head -n1`
		if [ ! -z ${old_file} ];then
			rm -f -- "${LOG_BAK_DIR}/logdump-${old_file}"
		else
			break
		fi
		log_package_size=`du -cm ${LOG_PACKAGE} | grep total | cut -f1`
		if [ -d ${LOG_BAK_DIR}/logdump ];then
			pushd ${LOG_BAK_DIR}/logdump > /dev/null
			logdump_dir_size=`du -sm | awk '{print $1}'`
			popd > /dev/null
		else
			logdump_dir_size=0
		fi
		((total_size=${log_package_size}+${logdump_dir_size}))
	done
	return 0
}


################################################################################
# Function Name: main
# Description: do check delete config
# Parameter: 1.log save dir
#            2.logdump package max save days
#            3.logdump package max save size
################################################################################
function main()
{
	LOG_BAK_DIR="${1}"
	LOG_SAVE_DAYS="${2}"
	LOG_SAVE_SIZE="${3}"

	if [ ! -d ${LOG_BAK_DIR} ];then
		return 0
	fi
	LOG_PACKAGE="${LOG_BAK_DIR}/logdump*.tar.gz"

	LOG_SAVE_DAYS=`echo $LOG_SAVE_DAYS | sed s/^0*//g`
	if (echo "${LOG_SAVE_DAYS}" | grep -q "^[[:digit:]]\{1,\}") && [ "${LOG_SAVE_DAYS}" -ne 0 ] ;then
		fn_delete_by_day
	fi
	LOG_SAVE_SIZE=`echo $LOG_SAVE_SIZE | sed s/^0*//g`
	if (echo "${LOG_SAVE_SIZE}" | grep -q "^[[:digit:]]\{1,\}") && [ "${LOG_SAVE_SIZE}" -ne 0 ];then
		fn_delete_by_size
	fi

}
main "$@"
exit $?
