#!/bin/bash
# Copyright (c) 2022 xFusion Digital Technologies Co., Ltd. All rights reserved.
# Description: log public include file, define the base functions
# File Name: esyslog_log_base_dump.sh
# Author: miaokaibo
# Create: 2022-04-15
source /opt/esyslog/esyslog_log_source_dump.sh
source /opt/esyslog/esyslog_log_common.sh

################################################################################
# Function Name: fn_check_if_mount
# Description: check the directory whether has mounted on the disk
# Parameter: parameter one, the directory that needs to be checked.
# output：none
# Return: 0-the dir has mounted on the disk, 1-the dir has not mounted
################################################################################
fn_check_if_mount()
{
	local check_dir="${1}"
	local mount_point=""
	local is_mount=""
	local all_mount_output=""
	local disk_mount_point_list=""
	local line=""
	local dev_name=""

	#create tmp file
	all_mount_output=$(OS_CREATE_TMP_FILE "/tmp/disk_mount")
	if [ $? -eq 1 ];then
		rm -rf "${all_mount_output}"
		return 1
	fi

	# get the all devices that has mounted in the system
	mount | grep ^"/dev" 2>/dev/null > ${all_mount_output}
	if [ ! -s "${all_mount_output}" ];then
		rm -rf "${all_mount_output}"
		return 1
	fi

	#create tmp file
	disk_mount_point_list=$(OS_CREATE_TMP_FILE "/tmp/mount_point")
	if [ $? -eq 1 ];then
		rm -rf "${disk_mount_point_list}"
		rm -rf "${all_mount_output}"
		return 1
	fi

	while read line
	do
		dev_name=$(echo "${line}" | awk '{print $1}')
		# get all disk mount points
		if [ -b "${dev_name}" ] && [ -z $(echo "${dev_name}" | egrep -w "/dev/(ram|loop)[0-9]{0,}") ];then
			echo "${line}" | awk '{print $3}' >> ${disk_mount_point_list}
		fi
	done < ${all_mount_output}

	# check the check_dir if is a part of mount_point
	while read mount_point
	do
		if [ -d "${mount_point}" -a -n "$(echo ${check_dir} | grep ^${mount_point})" ];then
			if [ "${mount_point}" == "${check_dir}" -o "${mount_point}" == "/" -o -n "$(echo ${check_dir#$mount_point} | grep ^/)" ];then
				rm -rf "${all_mount_output}"
				rm -rf "${disk_mount_point_list}"
				return 0
			fi
		fi
	done < ${disk_mount_point_list}

	rm -rf "${all_mount_output}"
	rm -rf "${disk_mount_point_list}"
	return 1
}

#################################################################################
# Function Name: fn_cal_time
# Description: calculate the real time in second unit by parsing the $1 file
#              which is the output of time command.
# Parameter: parameter one, the file name, the file is the output of time command
# output：the real time in second unit
# Return: none.
#################################################################################
fn_cal_time()
{
	local time_file="$1"
	local tmp_file=""
	local min=0
	local sec=0

	#create tmp file
	tmp_file=$(OS_CREATE_TMP_FILE "/tmp/tmp_real_time")
	if [ $? -eq 1 ];then
		rm -rf "${tmp_file}"
		return 1
	fi

	cat "${time_file}" | grep "real" | awk '{print $2}' 2>/dev/null > "${tmp_file}"
	min=$(cat "${tmp_file}" | awk -F"m" '{print $1}' 2>/dev/null)
	sec=$(cat "${tmp_file}" | awk -F"m" '{print $2}' | awk -F"." '{print $1}' 2>/dev/null)
	rm -rf "${tmp_file}"

	echo "$((min*60+sec))"
}

