#!/bin/bash
# Copyright (c) 2022 xFusion Digital Technologies Co., Ltd. All rights reserved.
# Description: log public include file, define the base functions
# Author: miaokaibo
# Create: 2022-04-15

# **************************************************************************** #
# Function Name: OS_CREATE_TMP_FILE
# Description: Create a secure tmp file
# Parameter: tmp file
# Return: 0-succ, 1-failed
# **************************************************************************** #
OS_CREATE_TMP_FILE()
{
        local file_name=$1
        local tmp_file=""
        local save_mask=$(umask)

        umask 077
        tmp_file=$(mktemp "${file_name}_XXXXXXXXXX")
        if [ $? -ne 0 ]
        then
                logger "mktemp file ${file_name} fail."
                umask "${save_mask}"
                return 1
        fi
        umask "${save_mask}"
        echo "${tmp_file}"
        return 0
}

# **************************************************************************** #
# Function Name: OS_CREATE_TMP_DIR
# Description: Create a secure tmp dir
# Parameter: tmp dir
# Return: 0-succ, 1-failed
# **************************************************************************** #
OS_CREATE_TMP_DIR()
{
        local dir_name=$1
        local tmp_dir=""
        local save_mask=$(umask)

        umask 077
        tmp_dir=$(mktemp -d "${dir_name}_XXXXXXXXXX")
        if [ $? -ne 0 ]
        then
                logger "mktemp dir ${dir_name} fail."
                umask "${save_mask}"
                return 1
        fi
        umask "${save_mask}"
        echo "${tmp_dir}"
        return 0
}
