#!/bin/bash
# Copyright (c) 2022 xFusion Digital Technologies Co., Ltd. All rights reserved.
# Description: split log, move log to temp dir, and check the log partition in time.
# File Name: esyslog_log_rsyslog_dump.sh
# Input: It is a character string, has two values.
#		1. source file that need to be rotated and compressed and saved to disk.
#		2. source file dir
# Author: miaokaibo
# Create: 2022-04-15

source /opt/esyslog/esyslog_log_dump_common.sh

umask 0027

cur_script_name="${BASH_SOURCE}"

################################################################################
# Function Name: fn_rsyslog_rotatelog
# Description: rotate the source log, mv log to logdump
# Parameter: parameter one, the source log file name which are needed to be rotated
#			parameter two, src log dir
#			parameter three, logrotate config
################################################################################
function fn_rsyslog_rotatelog()
{
	local src_file_name="${1##*/}"
	local src_file="$1"
	local src_dir="$2"
	local rotate_conf="$3"
	local diskflag="$4"
	local dump_dir=""
	local tmp_file=""
	local rotate_date=`date "+%Y%m%d%H%M%S"`
	local tmp_logdump_dir="${LOG_BAK_DIR}"
	local ret=0

	if [ $diskflag -eq 1 ];then
		tmp_logdump_dir="${LOG_DIR}"
		if [ -f "${LOG_BAK_DIR}/logdump.1" ];then
			rm "${LOG_BAK_DIR}"/logdump.1 -f 2>&1 | logger -it "esyslog_log_rsyslog_dump.sh" -p warning
			[ ${PIPESTATUS[0]} -ne 0 ] && return 1
		fi
		if [ ! -d "${LOG_BAK_DIR}/logdump" ];then
                        if [ -f "${LOG_BAK_DIR}/logdump" ];then
				rm "${LOG_BAK_DIR}"/logdump -f 2>&1 | logger -it "esyslog_log_rsyslog_dump.sh" -p warning
				[ ${PIPESTATUS[0]} -ne 0 ] && return 1
                        fi
			mkdir -p ${LOG_BAK_DIR}/logdump -m 700 2>&1 | logger -it "esyslog_log_rsyslog_dump.sh" -p warning
			[ ${PIPESTATUS[0]} -ne 0 ] && return 1
                fi
	fi
	if [ -f "${tmp_logdump_dir}/logdump.1" ];then
		rm "${tmp_logdump_dir}"/logdump.1 -f 2>&1 | logger -it "esyslog_log_rsyslog_dump.sh" -p warning
		[ ${PIPESTATUS[0]} -ne 0 ] && return 1
	fi
        if [ ! -d "${tmp_logdump_dir}/logdump" ];then
                if [ -f "${tmp_logdump_dir}/logdump" ];then
                        rm "${tmp_logdump_dir}"/logdump -f 2>&1 | logger -it "esyslog_log_rsyslog_dump.sh" -p warning
			[ ${PIPESTATUS[0]} -ne 0 ] && return 1
                fi
                mkdir -p ${tmp_logdump_dir}/logdump -m 700 2>&1 | logger -it "esyslog_log_rsyslog_dump.sh" -p warning
		[ ${PIPESTATUS[0]} -ne 0 ] && return 1
        fi
	if [ ${src_dir} == "logdump" ];then
		tmp_file="${tmp_logdump_dir}/logdump/${src_file_name}.1-${rotate_date}"
		dump_dir="${tmp_logdump_dir}/logdump"

	else
		if [ ! -d "${tmp_logdump_dir}/logdump/${src_dir}" ];then
			if [ -f "${tmp_logdump_dir}/logdump/${src_dir}" ];then
				rm -f "${tmp_logdump_dir}"/logdump/${src_dir}
			fi
			mkdir -p ${tmp_logdump_dir}/logdump/${src_dir} -m 700
			chmod 700 `find ${tmp_logdump_dir}/logdump/${src_dir} -type d ! -perm 700` 2> /dev/null
		fi

		tmp_file="${tmp_logdump_dir}/logdump/${src_dir}/${src_file_name}.1-${rotate_date}"
		dump_dir="${tmp_logdump_dir}/logdump/${src_dir}"
	fi

	exist_file=`ls ${dump_dir} | grep ${tmp_file##*/}`
	if [ ! -z "${exist_file}" ];then
		max_rotate_id=`ls ${dump_dir} | grep "${src_file_name}\..*-${rotate_date}" | awk -F "." '{print $NF}'  | awk -F "-" '{print $1}' | sort -nr | head -n1`
		((new_id=${max_rotate_id}+1))
		tmp_file=${tmp_file%/*}/${src_file_name}.${new_id}-${rotate_date}
	fi
	if [ -z "${rotate_conf}" ];then
		mv ${src_file} ${tmp_file} 2>&1 | logger -it esyslog_log_rsyslog_dump.sh -p warning
		chmod 400 ${tmp_file}
		# 0->LOG_BAK_DIR!=/var/log; 1->LOG_BAK_DIR=/var/log
		dir_is_var_log "${LOG_BAK_DIR}"
		ret="$?"
		# Only if /var/log is in memory and LOG_BAK_DIR!=/var/log, Logs will be synchronized to disk.
		if [ ${diskflag} -eq 1 ] && [ ${ret} -eq 0 ];then
			${ESYSLOG_LOG_SAVE_TO_DISK} "${tmp_file}" "${LOG_BAK_DIR}/${dump_dir#${tmp_logdump_dir}}" "${LOG_BAK_DIR}" "${LOG_DUMP_LIMIT}" &
		fi

	else
		rm -f "${src_file}".tmp
		logrotate -f ${rotate_conf} >/dev/null 2>&1
		rotate_file_list_num=`ls ${src_file%/*} | grep "${src_file_name}\.[[:digit:]]\{1,\}$" | awk -F "." '{print $NF}' | sort -nr`
		for file_num in $rotate_file_list_num
		do
			exist_file=`ls ${dump_dir} | grep ${tmp_file##*/}`
			if [ ! -z "${exist_file}" ];then
				max_rotate_id=`ls ${dump_dir} | grep "${src_file_name}\..*-${rotate_date}" | awk -F "." '{print $NF}'  | awk -F "-" '{print $1}' | sort -nr | head -n1`
				((new_id=${max_rotate_id}+1))
				tmp_file=${tmp_file%/*}/${src_file_name}.${new_id}-${rotate_date}
			fi
			mv ${src_file}.${file_num} ${tmp_file} 2>&1 | logger -it esyslog_log_rsyslog_dump.sh -p warning
			chmod 400 ${tmp_file}
			# 0->LOG_BAK_DIR!=/var/log; 1->LOG_BAK_DIR=/var/log
			dir_is_var_log "${LOG_BAK_DIR}"
			ret="$?"
			# Only if /var/log is in memory and LOG_BAK_DIR!=/var/log, Logs will be synchronized to disk.
			if [ ${diskflag} -eq 1 ] && [ ${ret} -eq 0 ];then
				${ESYSLOG_LOG_SAVE_TO_DISK} "${tmp_file}" "${LOG_BAK_DIR}/${dump_dir#${tmp_logdump_dir}}" "${LOG_BAK_DIR}" "${LOG_DUMP_LIMIT}" &
			fi
			rotate_date=`date "+%Y%m%d%H%M%S"`
			tmp_file=${tmp_file%/*}/${src_file_name}.1-${rotate_date}
		done
	fi
	return 0
}

################################################################################
# Function Name: main
# Description: check source log, check reentry, rotate the source log, check
#			  log dir partition,rotated current logs.
# Parameter: It is a character string, has three values.
#		1. source file that need to be rotated and compressed and dump
#		2. source file directory structure.
#		3. logrotate config
################################################################################
function main()
{
	#parse parameter
	declare -a param_array
	param_array=($@)
	local src_file="${param_array[0]}"
	local src_dir="${param_array[1]}"
	local rotate_conf="${param_array[2]}"

	# define the local variables
	local cur_size=0
	local diskflag=0
	local ret=0

	#check the parameters
	if [ ! -f "${src_file}" ];then
		${G_LOG_PRINT_CMD} "err" "[${cur_script_name}] Error, src_file[${src_file}] is not exist."
		return 1
	fi

	if [ -z "${src_dir}" ];then
		${G_LOG_PRINT_CMD} "err" "[${cur_script_name}] Error, src_dir[${src_dir}] is null."
		return 1
	fi

	#get global values
	fn_get_values_from_oslogdump_conf
	[ -d "${LOG_BAK_DIR}" ] || mkdir -p "${LOG_BAK_DIR}" -m 700 2>&1 || logger -it  "${LOG_BAK_DIR}" esyslog_log_rsyslog_dump.sh create "${LOG_BAK_DIR}" fail -p warning
	if [ ! -d "${LOG_BAK_DIR}" ];then
		${G_LOG_PRINT_CMD} "err" "[${cur_script_name}] Error: "${LOG_BAK_DIR}" does not exist"
		exit 1
	fi
	# the shell can not be runned twice at the same time
	fn_flock check_disk.lock
	# check log partition if over the maximum
	fn_chk_log_partition ${LOG_BAK_DIR} ${LOG_BAK_DIR_MAX_USAGE} ${LOG_BAK_DIR_RESUME_USAGE}
	fn_unflock check_disk.lock

	# check if /var/log/ is in the memory, if it is, set variable 'diskflag' to 1.
	fn_check_if_mount $LOG_DIR
	if [ $? -eq 1 ];then
		diskflag=1
	fi

	fn_flock logdump.lock
	# rotate the src_file
	fn_rsyslog_rotatelog "${src_file}" "${src_dir}" "${rotate_conf}" "${diskflag}"
	[ $? -ne 0 ] && exit 1
	# 0->LOG_BAK_DIR!=/var/log; 1->LOG_BAK_DIR=/var/log
	dir_is_var_log "${LOG_BAK_DIR}"
	ret="$?"
	# /var/log in disk or /var/log is in memory and LOG_BAK_DIR=/var/log to compress log
	if [ ${diskflag} -eq 0 ] || ([ ${diskflag} -eq 1 ] && [ ${ret} -eq 1 ]);then
		# dump the log file to ${LOG_BAK_DIR}
		if [ -d ${LOG_BAK_DIR}/logdump ];then
			cur_size=`du -sm ${LOG_BAK_DIR}/logdump | awk '{print $1}'`
		fi
		if [ "${cur_size}" -gt "${LOG_DUMP_LIMIT}" ];then
			fn_dump_logfile "${LOG_BAK_DIR}"
		fi
	fi
	fn_unflock logdump.lock


	${DELETE_BY_DAY_OR_SIZE_SCRIPT} "${LOG_BAK_DIR}" "${LOG_SAVE_DAYS}" "${LOG_SAVE_SIZE}"
	if [ $? -ne 0 ];then
		${G_LOG_PRINT_CMD} "err" "[${cur_script_name}] Error [ ${DELETE_BY_DAY_OR_SIZE_SCRIPT} ${LOG_BAK_DIR} ${LOG_SAVE_DAYS} ${LOG_SAVE_SIZE} ] execute failed."
		return 1
	fi
	return 0
}

main "$@"
exit $?
